/* Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package frontend.sjava;

import osio.TextReader;
import compbase.*;
import frontend.SScanner;
import frontend.Language;

/**
 * SJava: administration of the SJava-language and access to the SJava-routines
 * 
 * @author S. Frenz
 * @version 100113 fixed modifier of struct dummy class
 *  version 091209 adopted movement of SScanner to frontend package
 *  version 091112 added support for unified vrbl init checks
 *  version 091102 added support for attaching source to Context
 *  version 090303 adopted changed osio package structure
 *  version 090207 added copyright notice
 *  version 070801 added test to avoid re-creation of STRUCT-dummy
 *  version 070114 reduced access level where possible
 *  version 061222 dynamized Marker
 *  version 061211 setting CALLED-flag of runtime methods
 *  version 061202 minimal changes
 *  version 061129 dynamized Magic
 *  version 061027 added support for STRUCT
 *  version 060628 added support for static compilation
 *  version 060607 initial version
 */

public class SJava extends Language {
  public final static String KEY_THIS="this", KEY_SUPER="super";
  public final static String KEY_MAGIC="MAGIC", KEY_MARKER="MARKER";
  public final static String KEY_STRUCT="STRUCT", KEY_DEFINE="DEFINE";
  private Context ctx;
  private SScanner s;
  private JParser p;
  private TextReader inText;
  
  protected static boolean isSpecialName(String name) {
    return name.equals(KEY_THIS) || name.equals(KEY_SUPER)
      || name.equals(KEY_MAGIC) || name.equals(KEY_MARKER)
      || name.equals(KEY_STRUCT) || name.equals(KEY_DEFINE);
  }
  
  protected static boolean checkVarInitType(Vrbl var, int checkModifier, Unit unitContext, Context ctx) {
    int cmpRes;
    boolean success=true;
    
    if (var.init!=null && (cmpRes=var.type.compareType(var.init, true, ctx))!=TypeRef.C_EQ
        && cmpRes!=TypeRef.C_TT) {
      if ((var.init.getBaseTypeConvertible()!=0 && (checkModifier&Modifier.MC_EXPCONV)==0)
          || (cmpRes!=TypeRef.C_NP && var.type.isIntfType())) {
        if ((var.init=ExEnc.getConvertedResolvedExpr(var.init, var.type, unitContext, ctx))==null) success=false;
      }
      else success=false;
    }
    if (success) return true;
    var.init.printPos(ctx, "incompatible or not implicitly convertible types");
    return false;
  }
  
  protected void init(Context iCtx) {
    StringList strTmp;
    Magic magic;
    
    //store required variables
    ctx=iCtx;
    inText=ctx.osio.getNewTextReader();
    s=new SScanner();
    p=new JParser();
    strTmp=new StringList(null, "java");
    strTmp.next=new StringList(null, "lang");
    //if there is another language that defines the default package, insert check here
    ctx.defUnits=ctx.root.searchSubPackage(strTmp, true);
    //add dummy-class for STRUCT-extension
    if (ctx.defUnits.searchUnit(KEY_STRUCT)==null) {
      ctx.structClass=new Clss(new QualID(strTmp, 0, -1, 0, 0), null,
          Modifier.M_ABSTR|Modifier.M_PUB|Modifier.M_STRUCT, -1, 0, 0);
      ctx.structClass.name=KEY_STRUCT;
      ctx.defUnits.addUnit(ctx.structClass);
      ctx.addUnit(ctx.structClass);
    }
    //initialize internal special functions
    ((magic=new Magic(ctx)).nextConfig=new Marker()).nextConfig=ctx.config;
    ctx.config=magic;
  }
  
  protected boolean fileCompetence(String name) {
    return name.endsWith(".java");
  }
  
  protected boolean scanparseFile(StringList fileName) {
    boolean success=true;
    
    //try to parse the file
    if (!inText.initData(fileName.str)) {
      ctx.out.print("Error opening input-file: ");
      ctx.out.println(fileName.str);
      success=false;
    }
    else {
      ctx.attachSource(fileName.tablePos, inText.data);
      s.init(inText, fileName.tablePos, ctx);
      if (!p.tokenize(s, fileName.tablePos, ctx)) {
        ctx.out.print("...parsing ");
        ctx.out.print(fileName.str);
        ctx.out.println(" failed");
        success=false;
      }
    }
    return success;
  }
}
